@@ -53,6 +53,54 @@ clutter_allocation_flags_get_type (void)
 
   return g_enum_type_id__volatile;
 }
+GType
+clutter_redraw_flags_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GFlagsValue values[] = {
+        { CLUTTER_REDRAW_CLIPPED_TO_BOX, "CLUTTER_REDRAW_CLIPPED_TO_BOX", "box" },
+        { CLUTTER_REDRAW_CLIPPED_TO_ALLOCATION, "CLUTTER_REDRAW_CLIPPED_TO_ALLOCATION", "allocation" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_flags_register_static (g_intern_static_string ("ClutterRedrawFlags"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
+
+/* enumerations from "./clutter-align-constraint.h" */
+#include "./clutter-align-constraint.h"
+
+GType
+clutter_align_axis_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_ALIGN_X_AXIS, "CLUTTER_ALIGN_X_AXIS", "x-axis" },
+        { CLUTTER_ALIGN_Y_AXIS, "CLUTTER_ALIGN_Y_AXIS", "y-axis" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("ClutterAlignAxis"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
 
 /* enumerations from "./clutter-animator.h" */
 #include "./clutter-animator.h"
@@ -80,6 +128,33 @@ clutter_interpolation_get_type (void)
   return g_enum_type_id__volatile;
 }
 
+/* enumerations from "./clutter-bind-constraint.h" */
+#include "./clutter-bind-constraint.h"
+
+GType
+clutter_bind_coordinate_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_BIND_X, "CLUTTER_BIND_X", "x" },
+        { CLUTTER_BIND_Y, "CLUTTER_BIND_Y", "y" },
+        { CLUTTER_BIND_Z, "CLUTTER_BIND_Z", "z" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("ClutterBindCoordinate"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
+
 /* enumerations from "./clutter-bin-layout.h" */
 #include "./clutter-bin-layout.h"
 
@@ -136,6 +211,33 @@ clutter_box_alignment_get_type (void)
   return g_enum_type_id__volatile;
 }
 
+/* enumerations from "./clutter-drag-action.h" */
+#include "./clutter-drag-action.h"
+
+GType
+clutter_drag_axis_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_DRAG_AXIS_NONE, "CLUTTER_DRAG_AXIS_NONE", "axis-none" },
+        { CLUTTER_DRAG_X_AXIS, "CLUTTER_DRAG_X_AXIS", "x-axis" },
+        { CLUTTER_DRAG_Y_AXIS, "CLUTTER_DRAG_Y_AXIS", "y-axis" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("ClutterDragAxis"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
+
 /* enumerations from "./clutter-event.h" */
 #include "./clutter-event.h"
 
@@ -301,6 +403,7 @@ clutter_feature_flags_get_type (void)
         { CLUTTER_FEATURE_SHADERS_GLSL, "CLUTTER_FEATURE_SHADERS_GLSL", "shaders-glsl" },
         { CLUTTER_FEATURE_OFFSCREEN, "CLUTTER_FEATURE_OFFSCREEN", "offscreen" },
         { CLUTTER_FEATURE_STAGE_MULTIPLE, "CLUTTER_FEATURE_STAGE_MULTIPLE", "stage-multiple" },
+        { CLUTTER_FEATURE_SWAP_EVENTS, "CLUTTER_FEATURE_SWAP_EVENTS", "swap-events" },
         { 0, NULL, NULL }
       };
       GType g_enum_type_id;
