@@ -15,6 +15,21 @@ GType clutter_actor_flags_get_type (void) G_GNUC_CONST;
 GType clutter_allocation_flags_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_ALLOCATION_FLAGS (clutter_allocation_flags_get_type())
 
+GType clutter_redraw_flags_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_REDRAW_FLAGS (clutter_redraw_flags_get_type())
+
+/* enumerations from "./clutter-align-constraint.h" */
+GType clutter_align_axis_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_ALIGN_AXIS (clutter_align_axis_get_type())
+
+/* enumerations from "./clutter-animator.h" */
+GType clutter_interpolation_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_INTERPOLATION (clutter_interpolation_get_type())
+
+/* enumerations from "./clutter-bind-constraint.h" */
+GType clutter_bind_coordinate_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_BIND_COORDINATE (clutter_bind_coordinate_get_type())
+
 /* enumerations from "./clutter-bin-layout.h" */
 GType clutter_bin_alignment_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_BIN_ALIGNMENT (clutter_bin_alignment_get_type())
@@ -23,6 +38,10 @@ GType clutter_bin_alignment_get_type (void) G_GNUC_CONST;
 GType clutter_box_alignment_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_BOX_ALIGNMENT (clutter_box_alignment_get_type())
 
+/* enumerations from "./clutter-drag-action.h" */
+GType clutter_drag_axis_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_DRAG_AXIS (clutter_drag_axis_get_type())
+
 /* enumerations from "./clutter-event.h" */
 GType clutter_modifier_type_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_MODIFIER_TYPE (clutter_modifier_type_get_type())
@@ -39,9 +58,6 @@ GType clutter_scroll_direction_get_type (void) G_GNUC_CONST;
 GType clutter_stage_state_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_STAGE_STATE (clutter_stage_state_get_type())
 
-GType clutter_input_device_type_get_type (void) G_GNUC_CONST;
-#define CLUTTER_TYPE_INPUT_DEVICE_TYPE (clutter_input_device_type_get_type())
-
 /* enumerations from "./clutter-feature.h" */
 GType clutter_feature_flags_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_FEATURE_FLAGS (clutter_feature_flags_get_type())
@@ -50,6 +66,10 @@ GType clutter_feature_flags_get_type (void) G_GNUC_CONST;
 GType clutter_flow_orientation_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_FLOW_ORIENTATION (clutter_flow_orientation_get_type())
 
+/* enumerations from "./clutter-input-device.h" */
+GType clutter_input_device_type_get_type (void) G_GNUC_CONST;
+#define CLUTTER_TYPE_INPUT_DEVICE_TYPE (clutter_input_device_type_get_type())
+
 /* enumerations from "./clutter-main.h" */
 GType clutter_init_error_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_INIT_ERROR (clutter_init_error_get_type())
