@@ -22,7 +22,6 @@
 #define g_marshal_value_peek_boxed(v)    g_value_get_boxed (v)
 #define g_marshal_value_peek_pointer(v)  g_value_get_pointer (v)
 #define g_marshal_value_peek_object(v)   g_value_get_object (v)
-#define g_marshal_value_peek_variant(v)  g_value_get_variant (v)
 #else /* !G_ENABLE_DEBUG */
 /* WARNING: This code accesses GValues directly, which is UNSUPPORTED API.
  *          Do not access GValues directly in your code. Instead, use the
@@ -46,7 +45,6 @@
 #define g_marshal_value_peek_boxed(v)    (v)->data[0].v_pointer
 #define g_marshal_value_peek_pointer(v)  (v)->data[0].v_pointer
 #define g_marshal_value_peek_object(v)   (v)->data[0].v_pointer
-#define g_marshal_value_peek_variant(v)  (v)->data[0].v_pointer
 #endif /* !G_ENABLE_DEBUG */
 
 
@@ -405,7 +403,89 @@ clutter_marshal_VOID__INT_INT_INT_INT (GClosure     *closure,
 
 /* VOID:OBJECT (./clutter-marshal.list:12) */
 
-/* VOID:OBJECT,OBJECT,PARAM (./clutter-marshal.list:13) */
+/* VOID:OBJECT,FLOAT,FLOAT (./clutter-marshal.list:13) */
+void
+clutter_marshal_VOID__OBJECT_FLOAT_FLOAT (GClosure     *closure,
+                                          GValue       *return_value G_GNUC_UNUSED,
+                                          guint         n_param_values,
+                                          const GValue *param_values,
+                                          gpointer      invocation_hint G_GNUC_UNUSED,
+                                          gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__OBJECT_FLOAT_FLOAT) (gpointer     data1,
+                                                         gpointer     arg_1,
+                                                         gfloat       arg_2,
+                                                         gfloat       arg_3,
+                                                         gpointer     data2);
+  register GMarshalFunc_VOID__OBJECT_FLOAT_FLOAT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__OBJECT_FLOAT_FLOAT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_object (param_values + 1),
+            g_marshal_value_peek_float (param_values + 2),
+            g_marshal_value_peek_float (param_values + 3),
+            data2);
+}
+
+/* VOID:OBJECT,FLOAT,FLOAT,INT,FLAGS (./clutter-marshal.list:14) */
+void
+clutter_marshal_VOID__OBJECT_FLOAT_FLOAT_INT_FLAGS (GClosure     *closure,
+                                                    GValue       *return_value G_GNUC_UNUSED,
+                                                    guint         n_param_values,
+                                                    const GValue *param_values,
+                                                    gpointer      invocation_hint G_GNUC_UNUSED,
+                                                    gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__OBJECT_FLOAT_FLOAT_INT_FLAGS) (gpointer     data1,
+                                                                   gpointer     arg_1,
+                                                                   gfloat       arg_2,
+                                                                   gfloat       arg_3,
+                                                                   gint         arg_4,
+                                                                   guint        arg_5,
+                                                                   gpointer     data2);
+  register GMarshalFunc_VOID__OBJECT_FLOAT_FLOAT_INT_FLAGS callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 6);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__OBJECT_FLOAT_FLOAT_INT_FLAGS) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_object (param_values + 1),
+            g_marshal_value_peek_float (param_values + 2),
+            g_marshal_value_peek_float (param_values + 3),
+            g_marshal_value_peek_int (param_values + 4),
+            g_marshal_value_peek_flags (param_values + 5),
+            data2);
+}
+
+/* VOID:OBJECT,OBJECT,PARAM (./clutter-marshal.list:15) */
 void
 clutter_marshal_VOID__OBJECT_OBJECT_PARAM (GClosure     *closure,
                                            GValue       *return_value G_GNUC_UNUSED,
@@ -444,7 +524,7 @@ clutter_marshal_VOID__OBJECT_OBJECT_PARAM (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,POINTER (./clutter-marshal.list:14) */
+/* VOID:OBJECT,POINTER (./clutter-marshal.list:16) */
 void
 clutter_marshal_VOID__OBJECT_POINTER (GClosure     *closure,
                                       GValue       *return_value G_GNUC_UNUSED,
@@ -481,9 +561,9 @@ clutter_marshal_VOID__OBJECT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER (./clutter-marshal.list:15) */
+/* VOID:POINTER (./clutter-marshal.list:17) */
 
-/* VOID:STRING,BOOLEAN,BOOLEAN (./clutter-marshal.list:16) */
+/* VOID:STRING,BOOLEAN,BOOLEAN (./clutter-marshal.list:18) */
 void
 clutter_marshal_VOID__STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
                                               GValue       *return_value G_GNUC_UNUSED,
@@ -522,7 +602,7 @@ clutter_marshal_VOID__STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,INT (./clutter-marshal.list:17) */
+/* VOID:STRING,INT (./clutter-marshal.list:19) */
 void
 clutter_marshal_VOID__STRING_INT (GClosure     *closure,
                                   GValue       *return_value G_GNUC_UNUSED,
@@ -559,11 +639,11 @@ clutter_marshal_VOID__STRING_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:UINT (./clutter-marshal.list:18) */
+/* VOID:UINT (./clutter-marshal.list:20) */
 
-/* VOID:VOID (./clutter-marshal.list:19) */
+/* VOID:VOID (./clutter-marshal.list:21) */
 
-/* VOID:STRING,INT,POINTER (./clutter-marshal.list:20) */
+/* VOID:STRING,INT,POINTER (./clutter-marshal.list:22) */
 void
 clutter_marshal_VOID__STRING_INT_POINTER (GClosure     *closure,
                                           GValue       *return_value G_GNUC_UNUSED,
