@@ -3,7 +3,7 @@
  *
  * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Copyright (C) 2007,2008,2009 Intel Corporation.
+ * Copyright (C) 2007,2008,2009,2010 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -38,16 +38,10 @@
 #include "cogl-framebuffer-private.h"
 #include "cogl-journal-private.h"
 #include "cogl-util.h"
-
-void _cogl_add_path_to_stencil_buffer (floatVec2    nodes_min,
-                                       floatVec2    nodes_max,
-                                       unsigned int path_size,
-                                       CoglPathNode *path,
-                                       gboolean      merge,
-                                       gboolean      need_clear);
+#include "cogl-path-private.h"
 
 typedef struct _CoglClipStack CoglClipStack;
-
+typedef struct _CoglClipStackEntry CoglClipStackEntry;
 typedef struct _CoglClipStackEntryRect CoglClipStackEntryRect;
 typedef struct _CoglClipStackEntryWindowRect CoglClipStackEntryWindowRect;
 typedef struct _CoglClipStackEntryPath CoglClipStackEntryPath;
@@ -59,14 +53,67 @@ typedef enum
     COGL_CLIP_STACK_PATH
   } CoglClipStackEntryType;
 
+/* A clip stack consists a list of entries. Each entry has a reference
+ * count and a link to its parent node. The child takes a reference on
+ * the parent and the CoglClipStack holds a reference to the top of
+ * the stack. There are no links back from the parent to the
+ * children. This allows stacks that have common ancestry to share the
+ * entries.
+ *
+ * For example, the following sequence of operations would generate
+ * the tree below:
+ *
+ * CoglHandle stack_a = _cogl_clip_stack_new ();
+ * _cogl_set_clip_stack (stack_a);
+ * cogl_clip_stack_push_rectangle (...);
+ * cogl_clip_stack_push_rectangle (...);
+ * CoglHandle stack_b = _cogl_clip_stack_copy (stack_a);
+ * cogl_clip_stack_push_from_path ();
+ * cogl_set_clip_stack (stack_b);
+ * cogl_clip_stack_push_window_rectangle (...);
+ *
+ *  stack_a
+ *         \ holds a ref to
+ *          +-----------+
+ *          | path node |
+ *          |ref count 1|
+ *          +-----------+
+ *                       \
+ *                        +-----------+  +-----------+
+ *       both tops hold   | rect node |  | rect node |
+ *       a ref to the     |ref count 2|--|ref count 1|
+ *       same rect node   +-----------+  +-----------+
+ *                       /
+ *          +-----------+
+ *          | win. rect |
+ *          |ref count 1|
+ *          +-----------+
+ *         / holds a ref to
+ *  stack_b
+ *
+ */
+
 struct _CoglClipStack
 {
-  GList *stack_top;
+  CoglHandleObject _parent;
+
+  CoglClipStackEntry *stack_top;
+};
+
+struct _CoglClipStackEntry
+{
+  CoglClipStackEntryType  type;
+
+  /* This will be null if there is no parent. If it is not null then
+     this node must be holding a reference to the parent */
+  CoglClipStackEntry     *parent;
+
+  unsigned int            ref_count;
 };
 
 struct _CoglClipStackEntryRect
 {
-  CoglClipStackEntryType type;
+  CoglClipStackEntry     _parent_data;
 
   /* The rectangle for this clip */
   float                  x0;
@@ -80,29 +127,32 @@ struct _CoglClipStackEntryRect
 
 struct _CoglClipStackEntryWindowRect
 {
-  CoglClipStackEntryType type;
+  CoglClipStackEntry     _parent_data;
 
-  /* The window space rectangle for this clip */
-  float                  x0;
-  float                  y0;
-  float                  x1;
-  float                  y1;
+  /* The window space rectangle for this clip. This is stored in
+     Cogl's coordinate space (ie, 0,0 is the top left) */
+  int                    x0;
+  int                    y0;
+  int                    x1;
+  int                    y1;
 };
 
 struct _CoglClipStackEntryPath
 {
-  CoglClipStackEntryType type;
+  CoglClipStackEntry     _parent_data;
 
   /* The matrix that was current when the clip was set */
   CoglMatrix             matrix;
 
-  floatVec2              path_nodes_min;
-  floatVec2              path_nodes_max;
-
-  unsigned int           path_size;
-  CoglPathNode           path[1];
+  CoglHandle             path;
 };
 
+static void _cogl_clip_stack_free (CoglClipStack *stack);
+
+COGL_HANDLE_DEFINE (ClipStack, clip_stack);
+
+#define COGL_CLIP_STACK(stack) ((CoglClipStack *) (stack))
+
 static void
 project_vertex (const CoglMatrix *modelview_matrix,
 		const CoglMatrix *projection_matrix,
@@ -342,390 +392,204 @@ disable_clip_planes (void)
   GE( glDisable (GL_CLIP_PLANE0) );
 }
 
-void
-cogl_clip_push_window_rectangle (int x_offset,
-	                         int y_offset,
-	                         int width,
-	                         int height)
+static gpointer
+_cogl_clip_stack_push_entry (CoglClipStack *clip_stack,
+                             size_t size,
+                             CoglClipStackEntryType type)
 {
-  CoglHandle framebuffer;
-  CoglClipStackState *clip_state;
-  CoglClipStack *stack;
-  int framebuffer_height;
+  CoglClipStackEntry *entry = g_slice_alloc (size);
+
+  /* The new entry starts with a ref count of 1 because the stack
+     holds a reference to it as it is the top entry */
+  entry->ref_count = 1;
+  entry->type = type;
+  entry->parent = clip_stack->stack_top;
+  clip_stack->stack_top = entry;
+
+  /* We don't need to take a reference to the parent from the entry
+     because the clip_stack would have had to reference the top of
+     the stack and we can just steal that */
+
+  return entry;
+}
+
+void
+_cogl_clip_stack_push_window_rectangle (CoglHandle handle,
+                                        int x_offset,
+                                        int y_offset,
+                                        int width,
+                                        int height)
+{
+  CoglClipStack *stack = COGL_CLIP_STACK (handle);
   CoglClipStackEntryWindowRect *entry;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  /* We don't log clip stack changes in the journal so we must flush
-   * it before making modifications */
-  _cogl_journal_flush ();
+  entry = _cogl_clip_stack_push_entry (stack,
+                                       sizeof (CoglClipStackEntryWindowRect),
+                                       COGL_CLIP_STACK_WINDOW_RECT);
 
-  framebuffer = _cogl_get_framebuffer ();
-  clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
-
-  stack = clip_state->stacks->data;
-
-  framebuffer_height = _cogl_framebuffer_get_height (framebuffer);
-
-  entry = g_slice_new (CoglClipStackEntryWindowRect);
-
-  /* We store the entry coordinates in OpenGL window coordinate space and so
-   * because Cogl defines the window origin to be top left but OpenGL defines
-   * it as bottom left we may need to convert the incoming coordinates.
-   *
-   * NB: Cogl forces all offscreen rendering to be done upside down so in this
-   * case no conversion is needed.
-   */
-  entry->type = COGL_CLIP_STACK_WINDOW_RECT;
   entry->x0 = x_offset;
   entry->x1 = x_offset + width;
-  if (cogl_is_offscreen (framebuffer))
-    {
-      entry->y0 = y_offset;
-      entry->y1 = y_offset + height;
-    }
-  else
-    {
-      entry->y0 = framebuffer_height - y_offset - height;
-      entry->y1 = framebuffer_height - y_offset;
-    }
-
-  /* Store it in the stack */
-  stack->stack_top = g_list_prepend (stack->stack_top, entry);
-
-  clip_state->stack_dirty = TRUE;
+  entry->y0 = y_offset;
+  entry->y1 = y_offset + height;
 }
 
-/* XXX: This is deprecated API */
 void
-cogl_clip_push_window_rect (float x_offset,
-	                    float y_offset,
-	                    float width,
-	                    float height)
-{
-  cogl_clip_push_window_rectangle (x_offset, y_offset, width, height);
-}
-
-/* Scale from OpenGL normalized device coordinates (ranging from -1 to 1)
- * to Cogl window/framebuffer coordinates (ranging from 0 to buffer-size) with
- * (0,0) being top left. */
-#define VIEWPORT_TRANSFORM_X(x, vp_origin_x, vp_width) \
-    (  ( ((x) + 1.0) * ((vp_width) / 2.0) ) + (vp_origin_x)  )
-/* Note: for Y we first flip all coordinates around the X axis while in
- * normalized device coodinates */
-#define VIEWPORT_TRANSFORM_Y(y, vp_origin_y, vp_height) \
-    (  ( ((-(y)) + 1.0) * ((vp_height) / 2.0) ) + (vp_origin_y)  )
-
-/* Transform a homogeneous vertex position from model space to Cogl
- * window coordinates (with 0,0 being top left) */
-static void
-transform_point (CoglMatrix *matrix_mv,
-                 CoglMatrix *matrix_p,
-                 float *viewport,
-                 float *x,
-                 float *y)
-{
-  float z = 0;
-  float w = 1;
-
-  /* Apply the modelview matrix transform */
-  cogl_matrix_transform_point (matrix_mv, x, y, &z, &w);
-
-  /* Apply the projection matrix transform */
-  cogl_matrix_transform_point (matrix_p, x, y, &z, &w);
-
-  /* Perform perspective division */
-  *x /= w;
-  *y /= w;
-
-  /* Apply viewport transform */
-  *x = VIEWPORT_TRANSFORM_X (*x, viewport[0], viewport[2]);
-  *y = VIEWPORT_TRANSFORM_Y (*y, viewport[1], viewport[3]);
-}
-
-#undef VIEWPORT_SCALE_X
-#undef VIEWPORT_SCALE_Y
-
-/* Try to push a rectangle given in object coordinates as a rectangle in window
- * coordinates instead of object coordinates */
-gboolean
-try_pushing_rect_as_window_rect (float x_1,
-	                         float y_1,
+_cogl_clip_stack_push_rectangle (CoglHandle handle,
+                                 float x_1,
+                                 float y_1,
                                  float x_2,
-                                 float y_2)
+                                 float y_2,
+                                 const CoglMatrix *modelview_matrix)
 {
-  CoglMatrix matrix;
-  CoglMatrix matrix_p;
-  float v[4];
-
-  cogl_get_modelview_matrix (&matrix);
-
-  /* If the modelview meets these constraints then a transformed rectangle
-   * should still be a rectangle when it reaches screen coordinates.
-   *
-   * FIXME: we are are making certain assumptions about the projection
-   * matrix a.t.m and should really be looking at the combined modelview
-   * and projection matrix.
-   * FIXME: we don't consider rotations that are a multiple of 90 degrees
-   * which could be quite common.
-   */
-  if (matrix.xy != 0 || matrix.xz != 0 ||
-      matrix.yx != 0 || matrix.yz != 0 ||
-      matrix.zx != 0 || matrix.zy != 0)
-    return FALSE;
-
-  cogl_get_projection_matrix (&matrix_p);
-  cogl_get_viewport (v);
-
-  transform_point (&matrix, &matrix_p, v, &x_1, &y_1);
-  transform_point (&matrix, &matrix_p, v, &x_2, &y_2);
-
-  /* Consider that the modelview matrix may flip the rectangle
-   * along the x or y axis... */
-#define SWAP(A,B) do { float tmp = B; B = A; A = tmp; } while (0)
-  if (x_1 > x_2)
-    SWAP (x_1, x_2);
-  if (y_1 > y_2)
-    SWAP (y_1, y_2);
-#undef SWAP
-
-  cogl_clip_push_window_rectangle (COGL_UTIL_NEARBYINT (x_1),
-                                   COGL_UTIL_NEARBYINT (y_1),
-                                   COGL_UTIL_NEARBYINT (x_2 - x_1),
-                                   COGL_UTIL_NEARBYINT (y_2 - y_1));
-  return TRUE;
-}
-
-void
-cogl_clip_push_rectangle (float x_1,
-                          float y_1,
-                          float x_2,
-                          float y_2)
-{
-  CoglHandle framebuffer;
-  CoglClipStackState *clip_state;
-  CoglClipStack *stack;
+  CoglClipStack *stack = COGL_CLIP_STACK (handle);
   CoglClipStackEntryRect *entry;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  /* We don't log clip stack changes in the journal so we must flush
-   * it before making modifications */
-  _cogl_journal_flush ();
-
-  /* Try and catch window space rectangles so we can redirect to
-   * cogl_clip_push_window_rect which will use scissoring. */
-  if (try_pushing_rect_as_window_rect (x_1, y_1, x_2, y_2))
-    return;
-
-  framebuffer = _cogl_get_framebuffer ();
-  clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
-
-  stack = clip_state->stacks->data;
-
-  entry = g_slice_new (CoglClipStackEntryRect);
-
   /* Make a new entry */
-  entry->type = COGL_CLIP_STACK_RECT;
+  entry = _cogl_clip_stack_push_entry (stack,
+                                       sizeof (CoglClipStackEntryRect),
+                                       COGL_CLIP_STACK_RECT);
+
   entry->x0 = x_1;
   entry->y0 = y_1;
   entry->x1 = x_2;
   entry->y1 = y_2;
 
-  cogl_get_modelview_matrix (&entry->matrix);
-
-  /* Store it in the stack */
-  stack->stack_top = g_list_prepend (stack->stack_top, entry);
-
-  clip_state->stack_dirty = TRUE;
-}
-
-/* XXX: Deprecated API */
-void
-cogl_clip_push (float x_offset,
-	        float y_offset,
-	        float width,
-	        float height)
-{
-  cogl_clip_push_rectangle (x_offset,
-                            y_offset,
-                            x_offset + width,
-                            y_offset + height);
+  entry->matrix = *modelview_matrix;
 }
 
 void
-cogl_clip_push_from_path_preserve (void)
+_cogl_clip_stack_push_from_path (CoglHandle handle,
+                                 CoglHandle path,
+                                 const CoglMatrix *modelview_matrix)
 {
-  CoglHandle framebuffer;
-  CoglClipStackState *clip_state;
-  CoglClipStack *stack;
+  CoglClipStack *stack = COGL_CLIP_STACK (handle);
   CoglClipStackEntryPath *entry;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+  entry = _cogl_clip_stack_push_entry (stack,
+                                       sizeof (CoglClipStackEntryPath),
+                                       COGL_CLIP_STACK_PATH);
 
-  /* We don't log clip stack changes in the journal so we must flush
-   * it before making modifications */
-  _cogl_journal_flush ();
+  entry->path = cogl_path_copy (path);
 
-  framebuffer = _cogl_get_framebuffer ();
-  clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
-
-  stack = clip_state->stacks->data;
-
-  entry = g_malloc (sizeof (CoglClipStackEntryPath)
-                    + sizeof (CoglPathNode) * (ctx->path_nodes->len - 1));
-
-  entry->type = COGL_CLIP_STACK_PATH;
-  entry->path_nodes_min = ctx->path_nodes_min;
-  entry->path_nodes_max = ctx->path_nodes_max;
-  entry->path_size = ctx->path_nodes->len;
-  memcpy (entry->path, ctx->path_nodes->data,
-          sizeof (CoglPathNode) * ctx->path_nodes->len);
-
-  cogl_get_modelview_matrix (&entry->matrix);
-
-  /* Store it in the stack */
-  stack->stack_top = g_list_prepend (stack->stack_top, entry);
-
-  clip_state->stack_dirty = TRUE;
-}
-
-void
-cogl_clip_push_from_path (void)
-{
-  cogl_clip_push_from_path_preserve ();
-
-  cogl_path_new ();
+  entry->matrix = *modelview_matrix;
 }
 
 static void
-_cogl_clip_pop_real (CoglClipStackState *clip_state)
+_cogl_clip_stack_entry_unref (CoglClipStackEntry *entry)
 {
-  CoglClipStack *stack;
-  gpointer entry;
-  CoglClipStackEntryType type;
+  /* Unref all of the entries until we hit the root of the list or the
+     entry still has a remaining reference */
+  while (entry && --entry->ref_count <= 0)
+    {
+      CoglClipStackEntry *parent = entry->parent;
 
-  /* We don't log clip stack changes in the journal so we must flush
-   * it before making modifications */
-  _cogl_journal_flush ();
+      switch (entry->type)
+        {
+        case COGL_CLIP_STACK_RECT:
+          g_slice_free1 (sizeof (CoglClipStackEntryRect), entry);
+          break;
 
-  stack = clip_state->stacks->data;
+        case COGL_CLIP_STACK_WINDOW_RECT:
+          g_slice_free1 (sizeof (CoglClipStackEntryWindowRect), entry);
+          break;
+
+        case COGL_CLIP_STACK_PATH:
+          cogl_handle_unref (((CoglClipStackEntryPath *) entry)->path);
+          g_slice_free1 (sizeof (CoglClipStackEntryPath), entry);
+          break;
+
+        default:
+          g_assert_not_reached ();
+        }
+
+      entry = parent;
+    }
+}
+
+void
+_cogl_clip_stack_pop (CoglHandle handle)
+{
+  CoglClipStack *stack = COGL_CLIP_STACK (handle);
+  CoglClipStackEntry *entry;
 
   g_return_if_fail (stack->stack_top != NULL);
 
-  entry = stack->stack_top->data;
-  type = *(CoglClipStackEntryType *) entry;
-
-  /* Remove the top entry from the stack */
-  if (type == COGL_CLIP_STACK_RECT)
-    g_slice_free (CoglClipStackEntryRect, entry);
-  else if (type == COGL_CLIP_STACK_WINDOW_RECT)
-    g_slice_free (CoglClipStackEntryWindowRect, entry);
-  else
-    g_free (entry);
-
-  stack->stack_top = g_list_delete_link (stack->stack_top,
-                                         stack->stack_top);
-
-  clip_state->stack_dirty = TRUE;
+  /* To pop we are moving the top of the stack to the old top's parent
+     node. The stack always needs to have a reference to the top entry
+     so we must take a reference to the new top. The stack would have
+     previously had a reference to the old top so we need to decrease
+     the ref count on that. We need to ref the new head first in case
+     this stack was the only thing referencing the old top. In that
+     case the call to _cogl_clip_stack_entry_unref will unref the
+     parent. */
+  entry = stack->stack_top;
+  stack->stack_top = entry->parent;
+  if (stack->stack_top)
+    stack->stack_top->ref_count++;
+  _cogl_clip_stack_entry_unref (entry);
 }
 
 void
-cogl_clip_pop (void)
+_cogl_clip_stack_flush (CoglHandle handle,
+                        gboolean *stencil_used_p)
 {
-  CoglHandle framebuffer;
-  CoglClipStackState *clip_state;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  framebuffer = _cogl_get_framebuffer ();
-  clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
-
-  _cogl_clip_pop_real (clip_state);
-}
-
-void
-_cogl_flush_clip_state (CoglClipStackState *clip_state)
-{
-  CoglClipStack *stack;
+  CoglClipStack *stack = COGL_CLIP_STACK (handle);
   int has_clip_planes;
   gboolean using_clip_planes = FALSE;
   gboolean using_stencil_buffer = FALSE;
-  GList *node;
   int scissor_x0 = 0;
   int scissor_y0 = 0;
   int scissor_x1 = G_MAXINT;
   int scissor_y1 = G_MAXINT;
   CoglMatrixStack *modelview_stack =
     _cogl_framebuffer_get_modelview_stack (_cogl_get_framebuffer ());
-
-  if (!clip_state->stack_dirty)
-    return;
-
-  /* The current primitive journal does not support tracking changes to the
-   * clip stack...  */
-  _cogl_journal_flush ();
-
-  /* XXX: the handling of clipping is quite complex. It may involve use of
-   * the Cogl Journal or other Cogl APIs which may end up recursively
-   * wanting to ensure the clip state is flushed. We need to ensure we
-   * don't recurse infinitely...
-   */
-  clip_state->stack_dirty = FALSE;
+  CoglClipStackEntry *entry;
 
   has_clip_planes = cogl_features_available (COGL_FEATURE_FOUR_CLIP_PLANES);
 
-  stack = clip_state->stacks->data;
-
-  clip_state->stencil_used = FALSE;
-
   disable_clip_planes ();
   disable_stencil_buffer ();
   GE (glDisable (GL_SCISSOR_TEST));
 
   /* If the stack is empty then there's nothing else to do */
   if (stack->stack_top == NULL)
-    return;
-
-  /* Find the bottom of the stack */
-  for (node = stack->stack_top; node->next; node = node->next);
-
-  /* Re-add every entry from the bottom of the stack up */
-  for (; node; node = node->prev)
     {
-      gpointer entry = node->data;
-      CoglClipStackEntryType type = *(CoglClipStackEntryType *) entry;
+      *stencil_used_p = FALSE;
+      return;
+    }
 
-      if (type == COGL_CLIP_STACK_PATH)
+  /* Add all of the entries. This will end up adding them in the
+     reverse order that they were specified but as all of the clips
+     are intersecting it should work out the same regardless of the
+     order */
+  for (entry = stack->stack_top; entry; entry = entry->parent)
+    {
+      if (entry->type == COGL_CLIP_STACK_PATH)
         {
-          CoglClipStackEntryPath *path = (CoglClipStackEntryPath *) entry;
+          CoglClipStackEntryPath *path_entry = (CoglClipStackEntryPath *) entry;
 
           _cogl_matrix_stack_push (modelview_stack);
-          _cogl_matrix_stack_set (modelview_stack, &path->matrix);
+          _cogl_matrix_stack_set (modelview_stack, &path_entry->matrix);
 
-          _cogl_add_path_to_stencil_buffer (path->path_nodes_min,
-                                            path->path_nodes_max,
-                                            path->path_size,
-                                            path->path,
+          _cogl_add_path_to_stencil_buffer (path_entry->path,
                                             using_stencil_buffer,
                                             TRUE);
 
           _cogl_matrix_stack_pop (modelview_stack);
 
           using_stencil_buffer = TRUE;
-
-          /* We can't use clip planes any more */
-          has_clip_planes = FALSE;
         }
-      else if (type == COGL_CLIP_STACK_RECT)
+      else if (entry->type == COGL_CLIP_STACK_RECT)
         {
           CoglClipStackEntryRect *rect = (CoglClipStackEntryRect *) entry;
 
           _cogl_matrix_stack_push (modelview_stack);
           _cogl_matrix_stack_set (modelview_stack, &rect->matrix);
 
-          /* If this is the first entry and we support clip planes then use
-             that instead */
+          /* If we support clip planes and we haven't already used
+             them then use that instead */
           if (has_clip_planes)
             {
               set_clip_planes (rect->x0,
@@ -752,7 +616,8 @@ _cogl_flush_clip_state (CoglClipStackState *clip_state)
         {
           /* Get the intersection of all window space rectangles in the clip
            * stack */
-          CoglClipStackEntryWindowRect *window_rect = entry;
+          CoglClipStackEntryWindowRect *window_rect =
+            (CoglClipStackEntryWindowRect *) entry;
           scissor_x0 = MAX (scissor_x0, window_rect->x0);
           scissor_y0 = MAX (scissor_y0, window_rect->y0);
           scissor_x1 = MIN (scissor_x1, window_rect->x1);
@@ -765,123 +630,89 @@ _cogl_flush_clip_state (CoglClipStackState *clip_state)
   if (using_clip_planes)
     enable_clip_planes ();
 
-  if (scissor_x0 >= scissor_x1 || scissor_y0 >= scissor_y1)
-    scissor_x0 = scissor_y0 = scissor_x1 = scissor_y1 = 0;
-
   if (!(scissor_x0 == 0 && scissor_y0 == 0 &&
         scissor_x1 == G_MAXINT && scissor_y1 == G_MAXINT))
     {
+      int scissor_y_start;
+
+      if (scissor_x0 >= scissor_x1 || scissor_y0 >= scissor_y1)
+        scissor_x0 = scissor_y0 = scissor_x1 = scissor_y1 = scissor_y_start = 0;
+      else
+        {
+          CoglHandle framebuffer = _cogl_get_framebuffer ();
+
+          /* We store the entry coordinates in Cogl coordinate space
+           * but OpenGL requires the window origin to be the bottom
+           * left so we may need to convert the incoming coordinates.
+           *
+           * NB: Cogl forces all offscreen rendering to be done upside
+           * down so in this case no conversion is needed.
+           */
+
+          if (cogl_is_offscreen (framebuffer))
+            scissor_y_start = scissor_y0;
+          else
+            {
+              int framebuffer_height =
+                _cogl_framebuffer_get_height (framebuffer);
+
+              scissor_y_start = framebuffer_height - scissor_y1;
+            }
+        }
+
       GE (glEnable (GL_SCISSOR_TEST));
-      GE (glScissor (scissor_x0, scissor_y0,
+      GE (glScissor (scissor_x0, scissor_y_start,
                      scissor_x1 - scissor_x0,
                      scissor_y1 - scissor_y0));
     }
 
-  clip_state->stencil_used = using_stencil_buffer;
+  *stencil_used_p = using_stencil_buffer;
 }
 
-/* XXX: This should never have been made public API! */
-void
-cogl_clip_ensure (void)
-{
-  CoglClipStackState *clip_state;
-
-  clip_state = _cogl_framebuffer_get_clip_state (_cogl_get_framebuffer ());
-  _cogl_flush_clip_state (clip_state);
-}
-
-static void
-_cogl_clip_stack_save_real (CoglClipStackState *clip_state)
+CoglHandle
+_cogl_clip_stack_new (void)
 {
   CoglClipStack *stack;
 
-  /* We don't log clip stack changes in the journal so we must flush
-   * it before making modifications */
-  _cogl_journal_flush ();
-
   stack = g_slice_new (CoglClipStack);
   stack->stack_top = NULL;
 
-  clip_state->stacks = g_slist_prepend (clip_state->stacks, stack);
-  clip_state->stack_dirty = TRUE;
+  return _cogl_clip_stack_handle_new (stack);
 }
 
 void
-cogl_clip_stack_save (void)
+_cogl_clip_stack_free (CoglClipStack *stack)
 {
-  CoglHandle framebuffer;
-  CoglClipStackState *clip_state;
+  /* We only need to unref the top node and this
+     should end up freeing all of the parents if need be */
+  if (stack->stack_top)
+    _cogl_clip_stack_entry_unref (stack->stack_top);
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  framebuffer = _cogl_get_framebuffer ();
-  clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
-
-  _cogl_clip_stack_save_real (clip_state);
-}
-
-static void
-_cogl_clip_stack_restore_real (CoglClipStackState *clip_state)
-{
-  CoglClipStack *stack;
-
-  g_return_if_fail (clip_state->stacks != NULL);
-
-  /* We don't log clip stack changes in the journal so we must flush
-   * it before making modifications */
-  _cogl_journal_flush ();
-
-  stack = clip_state->stacks->data;
-
-  /* Empty the current stack */
-  while (stack->stack_top)
-    _cogl_clip_pop_real (clip_state);
-
-  /* Revert to an old stack */
   g_slice_free (CoglClipStack, stack);
-  clip_state->stacks = g_slist_delete_link (clip_state->stacks,
-                                            clip_state->stacks);
-
-  clip_state->stack_dirty = TRUE;
 }
 
-void
-cogl_clip_stack_restore (void)
+CoglHandle
+_cogl_clip_stack_copy (CoglHandle handle)
 {
-  CoglHandle framebuffer;
-  CoglClipStackState *clip_state;
+  CoglHandle new_handle;
+  CoglClipStack *new_stack, *old_stack;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+  if (!cogl_is_clip_stack (handle))
+    return COGL_INVALID_HANDLE;
 
-  framebuffer = _cogl_get_framebuffer ();
-  clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
+  old_stack = COGL_CLIP_STACK (handle);
 
-  _cogl_clip_stack_restore_real (clip_state);
+  new_handle = _cogl_clip_stack_new ();
+  new_stack = COGL_CLIP_STACK (new_handle);
+
+  /* We can copy the stack by just referencing the other stack's
+     data. There's no need to implement copy-on-write because the
+     entries of the stack can't be modified. If the other stack pops
+     some entries off they will still be kept alive because this stack
+     holds a reference. */
+  new_stack->stack_top = old_stack->stack_top;
+  if (new_stack->stack_top)
+    new_stack->stack_top->ref_count++;
+
+  return new_handle;
 }
-
-void
-_cogl_clip_stack_state_init (CoglClipStackState *clip_state)
-{
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  clip_state->stacks = NULL;
-  clip_state->stack_dirty = TRUE;
-
-  /* Add an intial stack */
-  _cogl_clip_stack_save_real (clip_state);
-}
-
-void
-_cogl_clip_stack_state_destroy (CoglClipStackState *clip_state)
-{
-  /* Destroy all of the stacks */
-  while (clip_state->stacks)
-    _cogl_clip_stack_restore_real (clip_state);
-}
-
-void
-_cogl_clip_stack_state_dirty (CoglClipStackState *clip_state)
-{
-  clip_state->stack_dirty = TRUE;
-}
-
