@@ -31,6 +31,7 @@
 #include "cogl-journal-private.h"
 #include "cogl-material-private.h"
 #include "cogl-framebuffer-private.h"
+#include "cogl-path-private.h"
 
 #include <string.h>
 #include <math.h>
@@ -41,37 +42,85 @@
 #define glClientActiveTexture ctx->drv.pf_glClientActiveTexture
 #endif
 
+static void _cogl_path_free (CoglPath *path);
+
+COGL_HANDLE_DEFINE (Path, path);
+
+static void
+_cogl_path_data_unref (CoglPathData *data)
+{
+  if (--data->ref_count <= 0)
+    {
+      g_array_free (data->path_nodes, TRUE);
+
+      g_slice_free (CoglPathData, data);
+    }
+}
+
+static void
+_cogl_path_modify (CoglPath *path)
+{
+  /* This needs to be called whenever the path is about to be modified
+     to implement copy-on-write semantics */
+
+  /* If there is more than one path using the data then we need to
+     copy the data instead */
+  if (path->data->ref_count != 1)
+    {
+      CoglPathData *old_data = path->data;
+
+      path->data = g_slice_dup (CoglPathData, old_data);
+      path->data->path_nodes = g_array_new (FALSE, FALSE,
+                                            sizeof (CoglPathNode));
+      g_array_append_vals (path->data->path_nodes,
+                           old_data->path_nodes->data,
+                           old_data->path_nodes->len);
+
+      path->data->ref_count = 1;
+
+      _cogl_path_data_unref (old_data);
+    }
+}
+
 static void
 _cogl_path_add_node (gboolean new_sub_path,
 		     float x,
 		     float y)
 {
   CoglPathNode new_node;
+  CoglPath *path;
+  CoglPathData *data;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  path = COGL_PATH (ctx->current_path);
+
+  _cogl_path_modify (path);
+
+  data = path->data;
+
   new_node.x = x;
   new_node.y = y;
   new_node.path_size = 0;
 
-  if (new_sub_path || ctx->path_nodes->len == 0)
-    ctx->last_path = ctx->path_nodes->len;
+  if (new_sub_path || data->path_nodes->len == 0)
+    data->last_path = data->path_nodes->len;
 
-  g_array_append_val (ctx->path_nodes, new_node);
+  g_array_append_val (data->path_nodes, new_node);
 
-  g_array_index (ctx->path_nodes, CoglPathNode, ctx->last_path).path_size++;
+  g_array_index (data->path_nodes, CoglPathNode, data->last_path).path_size++;
 
-  if (ctx->path_nodes->len == 1)
+  if (data->path_nodes->len == 1)
     {
-      ctx->path_nodes_min.x = ctx->path_nodes_max.x = x;
-      ctx->path_nodes_min.y = ctx->path_nodes_max.y = y;
+      data->path_nodes_min.x = data->path_nodes_max.x = x;
+      data->path_nodes_min.y = data->path_nodes_max.y = y;
     }
   else
     {
-      if (x < ctx->path_nodes_min.x) ctx->path_nodes_min.x = x;
-      if (x > ctx->path_nodes_max.x) ctx->path_nodes_max.x = x;
-      if (y < ctx->path_nodes_min.y) ctx->path_nodes_min.y = y;
-      if (y > ctx->path_nodes_max.y) ctx->path_nodes_max.y = y;
+      if (x < data->path_nodes_min.x) data->path_nodes_min.x = x;
+      if (x > data->path_nodes_max.x) data->path_nodes_max.x = x;
+      if (y < data->path_nodes_min.y) data->path_nodes_min.y = y;
+      if (y > data->path_nodes_max.y) data->path_nodes_max.y = y;
     }
 }
 
@@ -80,10 +129,13 @@ _cogl_path_stroke_nodes (void)
 {
   unsigned int   path_start = 0;
   unsigned long  enable_flags = COGL_ENABLE_VERTEX_ARRAY;
+  CoglPathData  *data;
   CoglMaterialFlushOptions options;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  data = COGL_PATH (ctx->current_path)->data;
+
   _cogl_journal_flush ();
 
   /* NB: _cogl_framebuffer_flush_state may disrupt various state (such
@@ -92,7 +144,7 @@ _cogl_path_stroke_nodes (void)
   _cogl_framebuffer_flush_state (_cogl_get_framebuffer (), 0);
 
   enable_flags |= _cogl_material_get_cogl_enable_flags (ctx->source_material);
-  cogl_enable (enable_flags);
+  _cogl_enable (enable_flags);
 
   options.flags = COGL_MATERIAL_FLUSH_DISABLE_MASK;
   /* disable all texture layers */
@@ -100,17 +152,15 @@ _cogl_path_stroke_nodes (void)
 
   _cogl_material_flush_gl_state (ctx->source_material, &options);
 
-  while (path_start < ctx->path_nodes->len)
+  while (path_start < data->path_nodes->len)
     {
-      CoglPathNode *path = &g_array_index (ctx->path_nodes, CoglPathNode,
+      CoglPathNode *node = &g_array_index (data->path_nodes, CoglPathNode,
                                            path_start);
 
-      GE( glVertexPointer (2, GL_FLOAT, sizeof (CoglPathNode),
-                           (guint8 *) path
-                           + G_STRUCT_OFFSET (CoglPathNode, x)) );
-      GE( glDrawArrays (GL_LINE_STRIP, 0, path->path_size) );
+      GE( glVertexPointer (2, GL_FLOAT, sizeof (CoglPathNode), &node->x) );
+      GE( glDrawArrays (GL_LINE_STRIP, 0, node->path_size) );
 
-      path_start += path->path_size;
+      path_start += node->path_size;
     }
 }
 
@@ -129,31 +179,28 @@ _cogl_path_get_bounds (floatVec2 nodes_min,
 }
 
 void
-_cogl_add_path_to_stencil_buffer (floatVec2 nodes_min,
-                                  floatVec2 nodes_max,
-                                  unsigned int  path_size,
-                                  CoglPathNode *path,
-                                  gboolean      merge,
-                                  gboolean      need_clear)
+_cogl_add_path_to_stencil_buffer (CoglHandle path_handle,
+                                  gboolean   merge,
+                                  gboolean   need_clear)
 {
   unsigned int     path_start = 0;
-  unsigned int     sub_path_num = 0;
   float            bounds_x;
   float            bounds_y;
   float            bounds_w;
   float            bounds_h;
   unsigned long    enable_flags = COGL_ENABLE_VERTEX_ARRAY;
   CoglHandle       prev_source;
-  int              i;
   CoglHandle       framebuffer = _cogl_get_framebuffer ();
   CoglMatrixStack *modelview_stack =
     _cogl_framebuffer_get_modelview_stack (framebuffer);
   CoglMatrixStack *projection_stack =
     _cogl_framebuffer_get_projection_stack (framebuffer);
-
+  CoglPath        *path;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  path = COGL_PATH (path_handle);
+
   /* We don't track changes to the stencil buffer in the journal
    * so we need to flush any batched geometry first */
   _cogl_journal_flush ();
@@ -171,9 +218,9 @@ _cogl_add_path_to_stencil_buffer (floatVec2 nodes_min,
 
   enable_flags |=
     _cogl_material_get_cogl_enable_flags (ctx->source_material);
-  cogl_enable (enable_flags);
+  _cogl_enable (enable_flags);
 
-  _cogl_path_get_bounds (nodes_min, nodes_max,
+  _cogl_path_get_bounds (path->data->path_nodes_min, path->data->path_nodes_max,
                          &bounds_x, &bounds_y, &bounds_w, &bounds_h);
 
   GE( glEnable (GL_STENCIL_TEST) );
@@ -207,7 +254,7 @@ _cogl_add_path_to_stencil_buffer (floatVec2 nodes_min,
            * enable flags */
           _cogl_matrix_stack_flush_to_gl (modelview_stack,
                                           COGL_MATRIX_MODELVIEW);
-          cogl_enable (enable_flags);
+          _cogl_enable (enable_flags);
         }
       GE (glStencilMask (1));
       GE (glStencilFunc (GL_LEQUAL, 0x1, 0x3));
@@ -215,45 +262,19 @@ _cogl_add_path_to_stencil_buffer (floatVec2 nodes_min,
 
   GE (glStencilOp (GL_INVERT, GL_INVERT, GL_INVERT));
 
-  for (i = 0; i < ctx->n_texcoord_arrays_enabled; i++)
+  /* Disable all client texture coordinate arrays */
+  _cogl_bitmask_clear_all (&ctx->temp_bitmask);
+  _cogl_disable_other_texcoord_arrays (&ctx->temp_bitmask);
+
+  while (path_start < path->data->path_nodes->len)
     {
-      GE (glClientActiveTexture (GL_TEXTURE0 + i));
-      GE (glDisableClientState (GL_TEXTURE_COORD_ARRAY));
-    }
-  ctx->n_texcoord_arrays_enabled = 0;
+      CoglPathNode *node =
+        &g_array_index (path->data->path_nodes, CoglPathNode, path_start);
 
-  while (path_start < path_size)
-    {
-      GE (glVertexPointer (2, GL_FLOAT, sizeof (CoglPathNode),
-                           (guint8 *) path
-                           + G_STRUCT_OFFSET (CoglPathNode, x)));
-      GE (glDrawArrays (GL_TRIANGLE_FAN, 0, path->path_size));
+      GE (glVertexPointer (2, GL_FLOAT, sizeof (CoglPathNode), &node->x));
+      GE (glDrawArrays (GL_TRIANGLE_FAN, 0, node->path_size));
 
-      if (sub_path_num > 0)
-        {
-          /* Union the two stencil buffers bits into the least
-             significant bit */
-          GE (glStencilMask (merge ? 6 : 3));
-          GE (glStencilOp (GL_ZERO, GL_REPLACE, GL_REPLACE));
-          cogl_rectangle (bounds_x, bounds_y,
-                          bounds_x + bounds_w, bounds_y + bounds_h);
-          /* Make sure the rectangle hits the stencil buffer before
-           * directly changing other GL state. */
-          _cogl_journal_flush ();
-          /* NB: The journal flushing may trash the modelview state and
-           * enable flags */
-          _cogl_matrix_stack_flush_to_gl (modelview_stack,
-                                          COGL_MATRIX_MODELVIEW);
-          cogl_enable (enable_flags);
-
-          GE (glStencilOp (GL_INVERT, GL_INVERT, GL_INVERT));
-        }
-
-      GE (glStencilMask (merge ? 4 : 2));
-
-      path_start += path->path_size;
-      path += path->path_size;
-      sub_path_num++;
+      path_start += node->path_size;
     }
 
   if (merge)
@@ -342,7 +363,7 @@ _cogl_path_fill_nodes_scanlines (CoglPathNode *path,
 
   _cogl_material_flush_gl_state (ctx->source_material, NULL);
 
-  cogl_enable (COGL_ENABLE_VERTEX_ARRAY
+  _cogl_enable (COGL_ENABLE_VERTEX_ARRAY
                | (ctx->color_alpha < 255 ? COGL_ENABLE_BLEND : 0));
 
   /* clear scanline intersection lists */
@@ -488,6 +509,7 @@ _cogl_path_fill_nodes_scanlines (CoglPathNode *path,
 static void
 _cogl_path_fill_nodes (void)
 {
+  CoglPathData *data;
   float bounds_x;
   float bounds_y;
   float bounds_w;
@@ -495,25 +517,23 @@ _cogl_path_fill_nodes (void)
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  _cogl_path_get_bounds (ctx->path_nodes_min, ctx->path_nodes_max,
+  data = COGL_PATH (ctx->current_path)->data;
+
+  _cogl_path_get_bounds (data->path_nodes_min, data->path_nodes_max,
                          &bounds_x, &bounds_y, &bounds_w, &bounds_h);
 
   if (G_LIKELY (!(cogl_debug_flags & COGL_DEBUG_FORCE_SCANLINE_PATHS)) &&
       cogl_features_available (COGL_FEATURE_STENCIL_BUFFER))
     {
       CoglHandle framebuffer;
-      CoglClipStackState *clip_state;
+      CoglClipState *clip_state;
 
       _cogl_journal_flush ();
 
       framebuffer = _cogl_get_framebuffer ();
       clip_state = _cogl_framebuffer_get_clip_state (framebuffer);
 
-      _cogl_add_path_to_stencil_buffer (ctx->path_nodes_min,
-                                        ctx->path_nodes_max,
-                                        ctx->path_nodes->len,
-                                        &g_array_index (ctx->path_nodes,
-                                                        CoglPathNode, 0),
+      _cogl_add_path_to_stencil_buffer (ctx->current_path,
                                         clip_state->stencil_used,
                                         FALSE);
 
@@ -529,23 +549,22 @@ _cogl_path_fill_nodes (void)
        * we call cogl_flush() to emtpy the journal.
        */
       cogl_flush ();
-      _cogl_clip_stack_state_dirty (clip_state);
+      _cogl_clip_state_dirty (clip_state);
     }
   else
     {
       unsigned int path_start = 0;
 
-      while (path_start < ctx->path_nodes->len)
+      while (path_start < data->path_nodes->len)
         {
-          CoglPathNode *path = &g_array_index (ctx->path_nodes, CoglPathNode,
+          CoglPathNode *node = &g_array_index (data->path_nodes, CoglPathNode,
                                                path_start);
 
-          _cogl_path_fill_nodes_scanlines (path,
-                                           path->path_size,
+          _cogl_path_fill_nodes_scanlines (node, node->path_size,
                                            bounds_x, bounds_y,
                                            bounds_w, bounds_h);
 
-          path_start += path->path_size;
+          path_start += node->path_size;
         }
     }
 }
@@ -563,7 +582,7 @@ cogl_path_fill_preserve (void)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  if (ctx->path_nodes->len == 0)
+  if (COGL_PATH (ctx->current_path)->data->path_nodes->len == 0)
     return;
 
   _cogl_path_fill_nodes ();
@@ -582,7 +601,7 @@ cogl_path_stroke_preserve (void)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  if (ctx->path_nodes->len == 0)
+  if (COGL_PATH (ctx->current_path)->data->path_nodes->len == 0)
     return;
 
   _cogl_path_stroke_nodes ();
@@ -592,57 +611,77 @@ void
 cogl_path_move_to (float x,
                    float y)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+  CoglPathData *data;
 
-  /* FIXME: handle multiple contours maybe? */
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   _cogl_path_add_node (TRUE, x, y);
 
-  ctx->path_start.x = x;
-  ctx->path_start.y = y;
+  data = COGL_PATH (ctx->current_path)->data;
 
-  ctx->path_pen = ctx->path_start;
+  data->path_start.x = x;
+  data->path_start.y = y;
+
+  data->path_pen = data->path_start;
 }
 
 void
 cogl_path_rel_move_to (float x,
                        float y)
 {
+  CoglPathData *data;
+
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  cogl_path_move_to (ctx->path_pen.x + x,
-                     ctx->path_pen.y + y);
+  data = COGL_PATH (ctx->current_path)->data;
+
+  cogl_path_move_to (data->path_pen.x + x,
+                     data->path_pen.y + y);
 }
 
 void
 cogl_path_line_to (float x,
                    float y)
 {
+  CoglPathData *data;
+
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   _cogl_path_add_node (FALSE, x, y);
 
-  ctx->path_pen.x = x;
-  ctx->path_pen.y = y;
+  data = COGL_PATH (ctx->current_path)->data;
+
+  data->path_pen.x = x;
+  data->path_pen.y = y;
 }
 
 void
 cogl_path_rel_line_to (float x,
                        float y)
 {
+  CoglPathData *data;
+
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  cogl_path_line_to (ctx->path_pen.x + x,
-                     ctx->path_pen.y + y);
+  data = COGL_PATH (ctx->current_path)->data;
+
+  cogl_path_line_to (data->path_pen.x + x,
+                     data->path_pen.y + y);
 }
 
 void
 cogl_path_close (void)
 {
+  CoglPath *path;
+
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  _cogl_path_add_node (FALSE, ctx->path_start.x, ctx->path_start.y);
-  ctx->path_pen = ctx->path_start;
+  path = COGL_PATH (ctx->current_path);
+
+  _cogl_path_add_node (FALSE, path->data->path_start.x,
+                       path->data->path_start.y);
+
+  path->data->path_pen = path->data->path_start;
 }
 
 void
@@ -650,7 +689,8 @@ cogl_path_new (void)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  g_array_set_size (ctx->path_nodes, 0);
+  cogl_handle_unref (ctx->current_path);
+  ctx->current_path = _cogl_path_new ();
 }
 
 void
@@ -780,19 +820,23 @@ cogl_path_arc (float center_x,
 }
 
 
-void
-cogl_path_arc_rel (float center_x,
-		   float center_y,
-		   float radius_x,
-		   float radius_y,
-		   float angle_1,
-		   float angle_2,
-		   float angle_step)
+static void
+_cogl_path_rel_arc (float center_x,
+                    float center_y,
+                    float radius_x,
+                    float radius_y,
+                    float angle_1,
+                    float angle_2,
+                    float angle_step)
 {
+  CoglPathData *data;
+
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  _cogl_path_arc (ctx->path_pen.x + center_x,
-	          ctx->path_pen.y + center_y,
+  data = COGL_PATH (ctx->current_path)->data;
+
+  _cogl_path_arc (data->path_pen.x + center_x,
+	          data->path_pen.y + center_y,
 	          radius_x,   radius_y,
 	          angle_1,    angle_2,
 	          angle_step, 0 /* no move */);
@@ -825,42 +869,45 @@ cogl_path_round_rectangle (float x_1,
                            float radius,
                            float arc_step)
 {
+  CoglPath *path;
   float inner_width = x_2 - x_1 - radius * 2;
   float inner_height = y_2 - y_1 - radius * 2;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  path = COGL_PATH (ctx->current_path);
+
   cogl_path_move_to (x_1, y_1 + radius);
-  cogl_path_arc_rel (radius, 0,
-		     radius, radius,
-		     180,
-		     270,
-		     arc_step);
+  _cogl_path_rel_arc (radius, 0,
+                      radius, radius,
+                      180,
+                      270,
+                      arc_step);
 
-  cogl_path_line_to       (ctx->path_pen.x + inner_width,
-                           ctx->path_pen.y);
-  cogl_path_arc_rel       (0, radius,
-			   radius, radius,
-			   -90,
-			   0,
-			   arc_step);
+  cogl_path_line_to       (path->data->path_pen.x + inner_width,
+                           path->data->path_pen.y);
+  _cogl_path_rel_arc      (0, radius,
+                           radius, radius,
+                           -90,
+                           0,
+                           arc_step);
 
-  cogl_path_line_to       (ctx->path_pen.x,
-                           ctx->path_pen.y + inner_height);
+  cogl_path_line_to       (path->data->path_pen.x,
+                           path->data->path_pen.y + inner_height);
 
-  cogl_path_arc_rel       (-radius, 0,
-			   radius, radius,
-			   0,
-			   90,
-			   arc_step);
+  _cogl_path_rel_arc      (-radius, 0,
+                           radius, radius,
+                           0,
+                           90,
+                           arc_step);
 
-  cogl_path_line_to       (ctx->path_pen.x - inner_width,
-                           ctx->path_pen.y);
-  cogl_path_arc_rel       (0, -radius,
-			   radius, radius,
-			   90,
-			   180,
-			   arc_step);
+  cogl_path_line_to       (path->data->path_pen.x - inner_width,
+                           path->data->path_pen.y);
+  _cogl_path_rel_arc      (0, -radius,
+                           radius, radius,
+                           90,
+                           180,
+                           arc_step);
 
   cogl_path_close ();
 }
@@ -970,11 +1017,14 @@ cogl_path_curve_to (float x_1,
                     float y_3)
 {
   CoglBezCubic cubic;
+  CoglPath *path;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  path = COGL_PATH (ctx->current_path);
+
   /* Prepare cubic curve */
-  cubic.p1 = ctx->path_pen;
+  cubic.p1 = path->data->path_pen;
   cubic.p2.x = x_1;
   cubic.p2.y = y_1;
   cubic.p3.x = x_2;
@@ -987,7 +1037,7 @@ cogl_path_curve_to (float x_1,
 
   /* Add last point */
   _cogl_path_add_node (FALSE, cubic.p4.x, cubic.p4.y);
-  ctx->path_pen = cubic.p4;
+  path->data->path_pen = cubic.p4;
 }
 
 void
@@ -998,16 +1048,84 @@ cogl_path_rel_curve_to (float x_1,
                         float x_3,
                         float y_3)
 {
+  CoglPathData *data;
+
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  cogl_path_curve_to (ctx->path_pen.x + x_1,
-                      ctx->path_pen.y + y_1,
-                      ctx->path_pen.x + x_2,
-                      ctx->path_pen.y + y_2,
-                      ctx->path_pen.x + x_3,
-                      ctx->path_pen.y + y_3);
+  data = COGL_PATH (ctx->current_path)->data;
+
+  cogl_path_curve_to (data->path_pen.x + x_1,
+                      data->path_pen.y + y_1,
+                      data->path_pen.x + x_2,
+                      data->path_pen.y + y_2,
+                      data->path_pen.x + x_3,
+                      data->path_pen.y + y_3);
 }
 
+CoglHandle
+cogl_path_get (void)
+{
+  _COGL_GET_CONTEXT (ctx, FALSE);
+
+  return ctx->current_path;
+}
+
+void
+cogl_path_set (CoglHandle handle)
+{
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  if (!cogl_is_path (handle))
+    return;
+
+  /* Reference the new handle first in case it is the same as the old
+     handle */
+  cogl_handle_ref (handle);
+  cogl_handle_unref (ctx->current_path);
+  ctx->current_path = handle;
+}
+
+CoglHandle
+_cogl_path_new (void)
+{
+  CoglPath *path;
+  CoglPathData *data;
+
+  path = g_slice_new (CoglPath);
+  data = path->data = g_slice_new (CoglPathData);
+
+  data->ref_count = 1;
+  data->path_nodes = g_array_new (FALSE, FALSE, sizeof (CoglPathNode));
+  data->last_path = 0;
+
+  return _cogl_path_handle_new (path);
+}
+
+CoglHandle
+cogl_path_copy (CoglHandle handle)
+{
+  CoglPath *old_path, *new_path;
+
+  _COGL_GET_CONTEXT (ctx, FALSE);
+
+  if (!cogl_is_path (handle))
+    return COGL_INVALID_HANDLE;
+
+  old_path = COGL_PATH (handle);
+
+  new_path = g_slice_new (CoglPath);
+  new_path->data = old_path->data;
+  new_path->data->ref_count++;
+
+  return _cogl_path_handle_new (new_path);
+}
+
+static void
+_cogl_path_free (CoglPath *path)
+{
+  _cogl_path_data_unref (path->data);
+  g_slice_free (CoglPath, path);
+}
 
 /* If second order beziers were needed the following code could
  * be re-enabled:
@@ -1086,12 +1204,15 @@ cogl_path_curve2_to (float x_1,
                      float x_2,
                      float y_2)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
+  CoglPath *path;
   CoglBezQuad quad;
 
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  path = COGL_PATH (ctx->current_path);
+
   /* Prepare quadratic curve */
-  quad.p1 = ctx->path_pen;
+  quad.p1 = path->data->path_pen;
   quad.p2.x = x_1;
   quad.p2.y = y_1;
   quad.p3.x = x_2;
@@ -1102,7 +1223,7 @@ cogl_path_curve2_to (float x_1,
 
   /* Add last point */
   _cogl_path_add_node (FALSE, quad.p3.x, quad.p3.y);
-  ctx->path_pen = quad.p3;
+  path->data->path_pen = quad.p3;
 }
 
 void
@@ -1111,11 +1232,16 @@ cogl_rel_curve2_to (float x_1,
                     float x_2,
                     float y_2)
 {
+  CoglPathData *data;
+
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  cogl_path_curve2_to (ctx->path_pen.x + x_1,
-                       ctx->path_pen.y + y_1,
-                       ctx->path_pen.x + x_2,
-                       ctx->path_pen.y + y_2);
+  data = COGL_PATH (ctx->current_path)->data;
+
+  cogl_path_curve2_to (data->path_pen.x + x_1,
+                       data->path_pen.y + y_1,
+                       data->path_pen.x + x_2,
+                       data->path_pen.y + y_2);
 }
+
 #endif
