@@ -224,12 +224,17 @@ _cogl_sub_texture_foreach_sub_texture_in_region (
 }
 
 static void
-_cogl_sub_texture_set_wrap_mode_parameter (CoglTexture *tex,
-                                           GLenum wrap_mode)
+_cogl_sub_texture_set_wrap_mode_parameters (CoglTexture *tex,
+                                            GLenum wrap_mode_s,
+                                            GLenum wrap_mode_t,
+                                            GLenum wrap_mode_r)
 {
   CoglSubTexture *sub_tex = COGL_SUB_TEXTURE (tex);
 
-  _cogl_texture_set_wrap_mode_parameter (sub_tex->full_texture, wrap_mode);
+  _cogl_texture_set_wrap_mode_parameters (sub_tex->full_texture,
+                                          wrap_mode_s,
+                                          wrap_mode_t,
+                                          wrap_mode_r);
 }
 
 static void
@@ -337,7 +342,7 @@ _cogl_sub_texture_transform_coords_to_gl (CoglTexture *tex,
   _cogl_texture_transform_coords_to_gl (sub_tex->full_texture, s, t);
 }
 
-static gboolean
+static CoglTransformResult
 _cogl_sub_texture_transform_quad_coords_to_gl (CoglTexture *tex,
                                                float *coords)
 {
@@ -348,13 +353,12 @@ _cogl_sub_texture_transform_quad_coords_to_gl (CoglTexture *tex,
      cogl-primitives will resort to manual repeating */
   for (i = 0; i < 4; i++)
     if (coords[i] < 0.0f || coords[i] > 1.0f)
-      return FALSE;
+      return COGL_TRANSFORM_SOFTWARE_REPEAT;
 
   _cogl_sub_texture_map_quad (sub_tex, coords);
 
-  _cogl_texture_transform_quad_coords_to_gl (sub_tex->full_texture, coords);
-
-  return TRUE;
+  return _cogl_texture_transform_quad_coords_to_gl (sub_tex->full_texture,
+                                                    coords);
 }
 
 static gboolean
@@ -548,7 +552,7 @@ cogl_sub_texture_vtable =
     _cogl_sub_texture_set_filters,
     _cogl_sub_texture_ensure_mipmaps,
     _cogl_sub_texture_ensure_non_quad_rendering,
-    _cogl_sub_texture_set_wrap_mode_parameter,
+    _cogl_sub_texture_set_wrap_mode_parameters,
     _cogl_sub_texture_get_format,
     _cogl_sub_texture_get_gl_format,
     _cogl_sub_texture_get_width,
