@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #if !defined(__COGL_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
@@ -32,6 +32,16 @@
 
 G_BEGIN_DECLS
 
+/* Some structures are meant to be opaque but they have public
+   definitions because we want the size to be public so they can be
+   allocated on the stack. This macro is used to ensure that users
+   don't accidentally access private members */
+#ifdef CLUTTER_COMPILATION
+#define COGL_PRIVATE(x) x
+#else
+#define COGL_PRIVATE(x) private_member_ ## x
+#endif
+
 /**
  * CoglHandle:
  *
@@ -49,7 +59,8 @@ typedef gpointer CoglHandle;
 #define COGL_INVALID_HANDLE NULL
 
 #define COGL_TYPE_HANDLE        (cogl_handle_get_type ())
-GType cogl_handle_get_type (void) G_GNUC_CONST;
+GType
+cogl_handle_get_type (void) G_GNUC_CONST;
 
 /**
  * cogl_handle_ref:
@@ -59,7 +70,8 @@ GType cogl_handle_get_type (void) G_GNUC_CONST;
  *
  * Returns: the handle, with its reference count increased
  */
-CoglHandle cogl_handle_ref   (CoglHandle handle);
+CoglHandle
+cogl_handle_ref (CoglHandle handle);
 
 /**
  * cogl_handle_unref:
@@ -68,7 +80,8 @@ CoglHandle cogl_handle_ref   (CoglHandle handle);
  * Drecreases the reference count of @handle by 1; if the reference
  * count reaches 0, the resources allocated by @handle will be freed
  */
-void       cogl_handle_unref (CoglHandle Handle);
+void
+cogl_handle_unref (CoglHandle Handle);
 
 /**
  * CoglFuncPtr:
@@ -87,7 +100,8 @@ typedef void (* CoglFuncPtr) (void);
 typedef gint32 CoglFixed;
 
 #define COGL_TYPE_FIXED         (cogl_fixed_get_type ())
-GType cogl_fixed_get_type (void) G_GNUC_CONST;
+GType
+cogl_fixed_get_type (void) G_GNUC_CONST;
 
 /**
  * CoglAngle:
@@ -195,6 +209,9 @@ typedef enum { /*< prefix=COGL_PIXEL_FORMAT >*/
  * @COGL_FEATURE_STENCIL_BUFFER: Stencil buffer support
  * @COGL_FEATURE_VBOS: VBO support
  * @COGL_FEATURE_PBOS: PBO support
+ * @COGL_FEATURE_UNSIGNED_INT_INDICES: Set if
+ *     %COGL_INDICES_TYPE_UNSIGNED_INT is supported in
+ *     cogl_vertex_buffer_indices_new().
  *
  * Flags for the supported features.
  *
@@ -213,7 +230,8 @@ typedef enum
   COGL_FEATURE_FOUR_CLIP_PLANES       = (1 << 9),
   COGL_FEATURE_STENCIL_BUFFER         = (1 << 10),
   COGL_FEATURE_VBOS		      = (1 << 11),
-  COGL_FEATURE_PBOS		      = (1 << 12)
+  COGL_FEATURE_PBOS		      = (1 << 12),
+  COGL_FEATURE_UNSIGNED_INT_INDICES   = (1 << 13)
 } CoglFeatureFlags;
 
 /**
@@ -243,17 +261,17 @@ typedef enum
 struct _CoglColor
 {
   /*< private >*/
-  guint8 red;
-  guint8 green;
-  guint8 blue;
+  guint8 COGL_PRIVATE (red);
+  guint8 COGL_PRIVATE (green);
+  guint8 COGL_PRIVATE (blue);
 
-  guint8 alpha;
+  guint8 COGL_PRIVATE (alpha);
 
   /* padding in case we want to change to floats at
    * some point */
-  guint32 padding0;
-  guint32 padding1;
-  guint32 padding2;
+  guint32 COGL_PRIVATE (padding0);
+  guint32 COGL_PRIVATE (padding1);
+  guint32 COGL_PRIVATE (padding2);
 };
 
 /**
@@ -283,8 +301,10 @@ struct _CoglTextureVertex
  *   the mipmap pyramid from the base level image whenever it is
  *   updated. The mipmaps are only generated when the texture is
  *   rendered with a mipmap filter so it should be free to leave out
- *   this flag when using other filtering modes.
+ *   this flag when using other filtering modes
  * @COGL_TEXTURE_NO_SLICING: Disables the slicing of the texture
+ * @COGL_TEXTURE_NO_ATLAS: Disables the insertion of the texture inside
+ *   the texture atlas used by Cogl
  *
  * Flags to pass to the cogl_texture_new_* family of functions.
  *
@@ -331,8 +351,27 @@ typedef enum {
   COGL_FOG_MODE_EXPONENTIAL_SQUARED
 } CoglFogMode;
 
+/**
+ * COGL_BLEND_STRING_ERROR:
+ *
+ * #GError domain for blend string parser errors
+ *
+ * Since: 1.0
+ */
 #define COGL_BLEND_STRING_ERROR (cogl_blend_string_error_quark ())
 
+/**
+ * CoglBlendStringError:
+ * @COGL_BLEND_STRING_ERROR_PARSE_ERROR: Generic parse error
+ * @COGL_BLEND_STRING_ERROR_ARGUMENT_PARSE_ERROR: Argument parse error
+ * @COGL_BLEND_STRING_ERROR_INVALID_ERROR: Internal parser error
+ * @COGL_BLEND_STRING_ERROR_GPU_UNSUPPORTED_ERROR: Blend string not
+ *   supported by the GPU
+ *
+ * Error enumeration for the blend strings parser
+ *
+ * Since: 1.0
+ */
 typedef enum { /*< prefix=COGL_BLEND_STRING_ERROR >*/
   COGL_BLEND_STRING_ERROR_PARSE_ERROR,
   COGL_BLEND_STRING_ERROR_ARGUMENT_PARSE_ERROR,
@@ -340,7 +379,8 @@ typedef enum { /*< prefix=COGL_BLEND_STRING_ERROR >*/
   COGL_BLEND_STRING_ERROR_GPU_UNSUPPORTED_ERROR
 } CoglBlendStringError;
 
-GQuark cogl_blend_string_error_quark (void);
+GQuark
+cogl_blend_string_error_quark (void);
 
 G_END_DECLS
 
