@@ -1518,10 +1518,8 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
   GLuint       generic_index = 0;
 #endif
   unsigned long enable_flags = 0;
-  int          max_texcoord_attrib_unit = -1;
   const GList *layers;
   guint32      fallback_layers = 0;
-  guint32      disable_layers = ~0;
   int          i;
   CoglMaterialFlushOptions options;
 
@@ -1530,6 +1528,13 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
   if (buffer->new_attributes)
     cogl_vertex_buffer_submit_real (buffer);
 
+  options.flags =
+    COGL_MATERIAL_FLUSH_FALLBACK_MASK;
+  memset (&options.wrap_mode_overrides, 0,
+          sizeof (options.wrap_mode_overrides));
+
+  _cogl_bitmask_clear_all (&ctx->temp_bitmask);
+
   for (tmp = buffer->submitted_vbos; tmp != NULL; tmp = tmp->next)
     {
       CoglVertexBufferVBO *cogl_vbo = tmp->data;
@@ -1592,9 +1597,8 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 				     gl_type,
 				     attribute->stride,
 				     pointer));
-              if (attribute->texture_unit > max_texcoord_attrib_unit)
-                max_texcoord_attrib_unit = attribute->texture_unit;
-              disable_layers &= ~(1 << attribute->texture_unit);
+              _cogl_bitmask_set (&ctx->temp_bitmask,
+                                 attribute->texture_unit, TRUE);
 	      break;
 	    case COGL_VERTEX_BUFFER_ATTRIB_FLAG_VERTEX_ARRAY:
 	      enable_flags |= COGL_ENABLE_VERTEX_ARRAY;
@@ -1632,7 +1636,7 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 
   layers = cogl_material_get_layers (ctx->source_material);
   for (tmp = (GList *)layers, i = 0;
-       tmp != NULL && i <= max_texcoord_attrib_unit;
+       tmp != NULL;
        tmp = tmp->next, i++)
     {
       CoglHandle layer = (CoglHandle)tmp->data;
@@ -1668,25 +1672,43 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
            */
           fallback_layers |= (1 << i);
         }
+
+      /* By default COGL_MATERIAL_WRAP_MODE_AUTOMATIC becomes
+         GL_CLAMP_TO_EDGE but we want GL_REPEAT to maintain
+         compatibility with older versions of Cogl so we'll override
+         it */
+      if (cogl_material_layer_get_wrap_mode_s (layer) ==
+          COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
+        {
+          options.wrap_mode_overrides.values[i].s =
+            COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
+          options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
+        }
+      if (cogl_material_layer_get_wrap_mode_t (layer) ==
+          COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
+        {
+          options.wrap_mode_overrides.values[i].t =
+            COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
+          options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
+        }
+      if (_cogl_material_layer_get_wrap_mode_r (layer) ==
+          COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
+        {
+          options.wrap_mode_overrides.values[i].r =
+            COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
+          options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
+        }
     }
 
-  for (i = max_texcoord_attrib_unit + 1; i < ctx->n_texcoord_arrays_enabled; i++)
-    {
-      GE (glClientActiveTexture (GL_TEXTURE0 + i));
-      GE (glDisableClientState (GL_TEXTURE_COORD_ARRAY));
-    }
-  ctx->n_texcoord_arrays_enabled = max_texcoord_attrib_unit + 1;
+  /* Disable any tex coord arrays that we didn't use */
+  _cogl_disable_other_texcoord_arrays (&ctx->temp_bitmask);
 
   /* NB: _cogl_framebuffer_flush_state may disrupt various state (such
    * as the material state) when flushing the clip stack, so should
    * always be done first when preparing to draw. */
   _cogl_framebuffer_flush_state (_cogl_get_framebuffer (), 0);
 
-  options.flags =
-    COGL_MATERIAL_FLUSH_FALLBACK_MASK |
-    COGL_MATERIAL_FLUSH_DISABLE_MASK;
   options.fallback_layers = fallback_layers;
-  options.disable_layers = disable_layers;
 
   _cogl_material_flush_gl_state (ctx->source_material, &options);
   enable_flags |= _cogl_material_get_cogl_enable_flags (ctx->source_material);
@@ -1694,7 +1716,7 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
   if (ctx->enable_backface_culling)
     enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
 
-  cogl_enable (enable_flags);
+  _cogl_enable (enable_flags);
   _cogl_flush_face_winding ();
 }
 
