@@ -3,7 +3,7 @@
  *
  * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Copyright (C) 2007,2008,2009 Intel Corporation.
+ * Copyright (C) 2007,2008,2009,2010 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -189,7 +189,7 @@ cogl_clear (const CoglColor *color, unsigned long buffers)
       return;
     }
 
-  glClear (gl_buffers);
+  GE (glClear (gl_buffers));
 
   /* This is a debugging variable used to visually display the quad
      batches from the journal. It is reset here to increase the
@@ -204,11 +204,11 @@ cogl_clear (const CoglColor *color, unsigned long buffers)
   COGL_NOTE (DRAW, "Clear end");
 }
 
-static inline gboolean
-cogl_toggle_flag (CoglContext *ctx,
-		  unsigned long new_flags,
-		  unsigned long flag,
-		  GLenum gl_flag)
+static gboolean
+toggle_flag (CoglContext *ctx,
+	     unsigned long new_flags,
+	     unsigned long flag,
+	     GLenum gl_flag)
 {
   /* Toggles and caches a single enable flag on or off
    * by comparing to current state
@@ -231,11 +231,11 @@ cogl_toggle_flag (CoglContext *ctx,
   return FALSE;
 }
 
-static inline gboolean
-cogl_toggle_client_flag (CoglContext *ctx,
-			 unsigned long new_flags,
-			 unsigned long flag,
-			 GLenum gl_flag)
+static gboolean
+toggle_client_flag (CoglContext *ctx,
+		    unsigned long new_flags,
+		    unsigned long flag,
+		    GLenum gl_flag)
 {
   /* Toggles and caches a single client-side enable flag
    * on or off by comparing to current state
@@ -259,32 +259,32 @@ cogl_toggle_client_flag (CoglContext *ctx,
 }
 
 void
-cogl_enable (unsigned long flags)
+_cogl_enable (unsigned long flags)
 {
   /* This function essentially caches glEnable state() in the
    * hope of lessening number GL traffic.
   */
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  cogl_toggle_flag (ctx, flags,
-                    COGL_ENABLE_BLEND,
-                    GL_BLEND);
+  toggle_flag (ctx, flags,
+               COGL_ENABLE_BLEND,
+               GL_BLEND);
 
-  cogl_toggle_flag (ctx, flags,
-                    COGL_ENABLE_BACKFACE_CULLING,
-                    GL_CULL_FACE);
+  toggle_flag (ctx, flags,
+               COGL_ENABLE_BACKFACE_CULLING,
+               GL_CULL_FACE);
 
-  cogl_toggle_client_flag (ctx, flags,
-			   COGL_ENABLE_VERTEX_ARRAY,
-			   GL_VERTEX_ARRAY);
+  toggle_client_flag (ctx, flags,
+		      COGL_ENABLE_VERTEX_ARRAY,
+		      GL_VERTEX_ARRAY);
 
-  cogl_toggle_client_flag (ctx, flags,
-			   COGL_ENABLE_COLOR_ARRAY,
-			   GL_COLOR_ARRAY);
+  toggle_client_flag (ctx, flags,
+		      COGL_ENABLE_COLOR_ARRAY,
+		      GL_COLOR_ARRAY);
 }
 
 unsigned long
-cogl_get_enable ()
+_cogl_get_enable (void)
 {
   _COGL_GET_CONTEXT (ctx, 0);
 
@@ -533,31 +533,21 @@ cogl_get_bitmasks (int *red,
                    int *blue,
                    int *alpha)
 {
-  GLint value;
+  CoglHandle framebuffer;
+
+  framebuffer = _cogl_get_framebuffer ();
 
   if (red)
-    {
-      GE( glGetIntegerv(GL_RED_BITS, &value) );
-      *red = value;
-    }
+    *red = cogl_framebuffer_get_red_bits (framebuffer);
 
   if (green)
-    {
-      GE( glGetIntegerv(GL_GREEN_BITS, &value) );
-      *green = value;
-    }
+    *green = cogl_framebuffer_get_green_bits (framebuffer);
 
   if (blue)
-    {
-      GE( glGetIntegerv(GL_BLUE_BITS, &value) );
-      *blue = value;
-    }
+    *blue = cogl_framebuffer_get_blue_bits (framebuffer);
 
   if (alpha)
-    {
-      GE( glGetIntegerv(GL_ALPHA_BITS, &value ) );
-      *alpha = value;
-    }
+    *alpha = cogl_framebuffer_get_alpha_bits (framebuffer);
 }
 
 void
@@ -773,12 +763,40 @@ cogl_read_pixels (int x,
     }
 }
 
+static void
+_cogl_disable_other_texcoord_arrays_cb (int texcoord_array_num, gpointer data)
+{
+  CoglContext *ctx = data;
+
+  GE (glClientActiveTexture (GL_TEXTURE0 + texcoord_array_num));
+  GE (glDisableClientState (GL_TEXTURE_COORD_ARRAY));
+}
+
+void
+_cogl_disable_other_texcoord_arrays (const CoglBitmask *mask)
+{
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  /* Set texcoord_arrays_to_disable to only contain the arrays we want
+     to disable */
+  _cogl_bitmask_clear_all (&ctx->texcoord_arrays_to_disable);
+  _cogl_bitmask_set_bits (&ctx->texcoord_arrays_to_disable,
+                          &ctx->texcoord_arrays_enabled);
+  _cogl_bitmask_clear_bits (&ctx->texcoord_arrays_to_disable, mask);
+
+  _cogl_bitmask_foreach (&ctx->texcoord_arrays_to_disable,
+                         _cogl_disable_other_texcoord_arrays_cb, ctx);
+
+  /* Update the mask of arrays that are enabled */
+  _cogl_bitmask_clear_bits (&ctx->texcoord_arrays_enabled,
+                            &ctx->texcoord_arrays_to_disable);
+}
+
 void
 cogl_begin_gl (void)
 {
   CoglMaterialFlushOptions options;
   unsigned long enable_flags = 0;
-  int i;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -816,22 +834,18 @@ cogl_begin_gl (void)
   _cogl_material_flush_gl_state (ctx->source_material, &options);
 
   /* FIXME: This api is a bit yukky, ideally it will be removed if we
-   * re-work the cogl_enable mechanism */
+   * re-work the _cogl_enable mechanism */
   enable_flags |= _cogl_material_get_cogl_enable_flags (ctx->source_material);
 
   if (ctx->enable_backface_culling)
     enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
 
-  cogl_enable (enable_flags);
+  _cogl_enable (enable_flags);
   _cogl_flush_face_winding ();
 
   /* Disable all client texture coordinate arrays */
-  for (i = 0; i < ctx->n_texcoord_arrays_enabled; i++)
-    {
-      GE (glClientActiveTexture (GL_TEXTURE0 + i));
-      GE (glDisableClientState (GL_TEXTURE_COORD_ARRAY));
-    }
-  ctx->n_texcoord_arrays_enabled = 0;
+  _cogl_bitmask_clear_all (&ctx->temp_bitmask);
+  _cogl_disable_other_texcoord_arrays (&ctx->temp_bitmask);
 }
 
 void
@@ -986,6 +1000,14 @@ cogl_rotate (float angle, float x, float y, float z)
   _cogl_matrix_stack_rotate (modelview_stack, angle, x, y, z);
 }
 
+void
+cogl_transform (const CoglMatrix *matrix)
+{
+  CoglMatrixStack *modelview_stack =
+    _cogl_framebuffer_get_modelview_stack (_cogl_get_framebuffer ());
+  _cogl_matrix_stack_multiply (modelview_stack, matrix);
+}
+
 void
 cogl_perspective (float fov_y,
 		  float aspect,
@@ -1084,7 +1106,7 @@ cogl_set_projection_matrix (CoglMatrix *matrix)
   _COGL_MATRIX_DEBUG_PRINT (matrix);
 }
 
-CoglClipStackState *
+CoglClipState *
 _cogl_get_clip_state (void)
 {
   CoglHandle framebuffer;
@@ -1098,3 +1120,61 @@ _cogl_driver_error_quark (void)
 {
   return g_quark_from_static_string ("cogl-driver-error-quark");
 }
+
+void
+cogl_set_source (CoglHandle material_handle)
+{
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  g_return_if_fail (cogl_is_material (material_handle));
+
+  if (ctx->source_material == material_handle)
+    return;
+
+  cogl_handle_ref (material_handle);
+
+  if (ctx->source_material)
+    cogl_handle_unref (ctx->source_material);
+
+  ctx->source_material = material_handle;
+}
+
+void
+cogl_set_source_texture (CoglHandle texture_handle)
+{
+  CoglColor white;
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  g_return_if_fail (texture_handle != COGL_INVALID_HANDLE);
+
+  cogl_material_set_layer (ctx->simple_material, 0, texture_handle);
+  cogl_color_set_from_4ub (&white, 0xff, 0xff, 0xff, 0xff);
+  cogl_material_set_color (ctx->simple_material, &white);
+  cogl_set_source (ctx->simple_material);
+}
+
+void
+cogl_set_source_color4ub (guint8 red,
+                          guint8 green,
+                          guint8 blue,
+                          guint8 alpha)
+{
+  CoglColor c = { 0, };
+
+  cogl_color_set_from_4ub (&c, red, green, blue, alpha);
+  cogl_set_source_color (&c);
+}
+
+void
+cogl_set_source_color4f (float red,
+                         float green,
+                         float blue,
+                         float alpha)
+{
+  CoglColor c = { 0, };
+
+  cogl_color_set_from_4f (&c, red, green, blue, alpha);
+  cogl_set_source_color (&c);
+}
+
