@@ -37,6 +37,7 @@
 #include <cogl/cogl-fixed.h>
 #include <cogl/cogl-material.h>
 #include <cogl/cogl-matrix.h>
+#include <cogl/cogl-vector.h>
 #include <cogl/cogl-offscreen.h>
 #include <cogl/cogl-primitives.h>
 #include <cogl/cogl-path.h>
@@ -52,6 +53,7 @@
 #if defined (COGL_ENABLE_EXPERIMENTAL_API)
 #include <cogl/cogl-buffer.h>
 #include <cogl/cogl-pixel-buffer.h>
+#include <cogl/cogl-vector.h>
 #endif
 
 G_BEGIN_DECLS
@@ -159,6 +161,58 @@ cogl_get_bitmasks (int *red,
                    int *blue,
                    int *alpha);
 
+/**
+ * cogl_framebuffer_get_red_bits:
+ * @framebuffer: a handle for a framebuffer
+ *
+ * Retrieves the number of red bits of @framebuffer
+ *
+ * Return value: the number of bits
+ *
+ * Since: 1.4
+ */
+int
+cogl_framebuffer_get_red_bits (CoglHandle framebuffer);
+
+/**
+ * cogl_framebuffer_get_green_bits:
+ * @framebuffer: a handle for a framebuffer
+ *
+ * Retrieves the number of green bits of @framebuffer
+ *
+ * Return value: the number of bits
+ *
+ * Since: 1.4
+ */
+int
+cogl_framebuffer_get_green_bits (CoglHandle framebuffer);
+
+/**
+ * cogl_framebuffer_get_blue_bits:
+ * @framebuffer: a handle for a framebuffer
+ *
+ * Retrieves the number of blue bits of @framebuffer
+ *
+ * Return value: the number of bits
+ *
+ * Since: 1.4
+ */
+int
+cogl_framebuffer_get_blue_bits (CoglHandle framebuffer);
+
+/**
+ * cogl_framebuffer_get_alpha_bits:
+ * @framebuffer: a handle for a framebuffer
+ *
+ * Retrieves the number of alpha bits of @framebuffer
+ *
+ * Return value: the number of bits
+ *
+ * Since: 1.4
+ */
+int
+cogl_framebuffer_get_alpha_bits (CoglHandle framebuffer);
+
 /**
  * cogl_perspective:
  * @fovy: Vertical of view angle in degrees.
@@ -347,6 +401,17 @@ cogl_rotate (float angle,
              float y,
              float z);
 
+/**
+ * cogl_transform:
+ * @matrix: the matrix to multiply with the current model-view
+ *
+ * Multiplies the current model-view matrix by the given matrix.
+ *
+ * Since: 1.4
+ */
+void
+cogl_transform (const CoglMatrix *matrix);
+
 /**
  * cogl_get_modelview_matrix:
  * @matrix: (out): return location for the model-view matrix
