@@ -435,14 +435,18 @@ clutter_x11_texture_pixmap_real_queue_damage_redraw (
    * actor coordinates...
    */
 
-  /* XXX: we don't care if we get an out of date allocation here because
-   * clutter_actor_queue_clipped_redraw knows to ignore the clip if the
-   * actor's allocation is invalid.
-   *
-   * This is noted because clutter_actor_get_allocation_box does some
-   * unnecessary work to support buggy code with a comment suggesting that
-   * it could be changed later which would be good for this use case!
+  /* Calling clutter_actor_get_allocation_box() is enormously expensive
+   * if the actor has an out-of-date allocation, since it triggers
+   * a full redraw. clutter_actor_queue_clipped_redraw() would redraw
+   * the whole stage anyways in that case, so just go ahead and do
+   * it here.
    */
+  if (!clutter_actor_has_allocation (self))
+    {
+      clutter_actor_queue_redraw (self);
+      return;
+    }
+
   clutter_actor_get_allocation_box (self, &allocation);
 
   g_object_get (self,
@@ -471,11 +475,6 @@ clutter_x11_texture_pixmap_init (ClutterX11TexturePixmap *self)
                                    CLUTTER_X11_TYPE_TEXTURE_PIXMAP,
                                    ClutterX11TexturePixmapPrivate);
 
-  g_signal_override_class_handler (
-            "queue-damage-redraw",
-            CLUTTER_X11_TYPE_TEXTURE_PIXMAP,
-            G_CALLBACK (clutter_x11_texture_pixmap_real_queue_damage_redraw));
-
   if (!check_extensions (self))
     {
       /* FIMXE: means display lacks needed extensions for at least auto.
@@ -814,17 +813,21 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
    * Since: 1.2
    */
   signals[QUEUE_DAMAGE_REDRAW] =
-      g_signal_new ("queue-damage-redraw",
-                    G_TYPE_FROM_CLASS (object_class),
-                    G_SIGNAL_RUN_FIRST,
-                    0,
-                    NULL, NULL,
-                    clutter_marshal_VOID__INT_INT_INT_INT,
-                    G_TYPE_NONE, 4,
-                    G_TYPE_INT,
-                    G_TYPE_INT,
-                    G_TYPE_INT,
-                    G_TYPE_INT);
+    g_signal_new (g_intern_static_string ("queue-damage-redraw"),
+                  G_TYPE_FROM_CLASS (object_class),
+                  G_SIGNAL_RUN_FIRST,
+                  0,
+                  NULL, NULL,
+                  clutter_marshal_VOID__INT_INT_INT_INT,
+                  G_TYPE_NONE, 4,
+                  G_TYPE_INT,
+                  G_TYPE_INT,
+                  G_TYPE_INT,
+                  G_TYPE_INT);
+
+  g_signal_override_class_handler ("queue-damage-redraw",
+                                   CLUTTER_X11_TYPE_TEXTURE_PIXMAP,
+                                   G_CALLBACK (clutter_x11_texture_pixmap_real_queue_damage_redraw));
 
   default_backend = clutter_get_default_backend ();
 
