@@ -3,6 +3,7 @@
 #include <clutter/clutter.h>
 
 #include <glib.h>
+#include <locale.h>
 #include <stdlib.h>
 
 #include "test-conform-common.h"
@@ -14,6 +15,8 @@ test_conform_skip_test (TestConformSimpleFixture *fixture,
   /* void */
 }
 
+static TestConformSharedState *shared_state = NULL;
+
 /* This is a bit of sugar for adding new conformance tests:
  *
  * - It adds an extern function definition just to save maintaining a header
@@ -56,11 +59,16 @@ test_conform_skip_test (TestConformSimpleFixture *fixture,
               test_conform_skip_test,                                   \
               test_conform_simple_fixture_teardown);    } G_STMT_END
 
-int
-main (int argc, char **argv)
+gchar *
+clutter_test_get_data_file (const gchar *filename)
 {
-  TestConformSharedState *shared_state = g_new0 (TestConformSharedState, 1);
+  return g_build_filename (TESTS_DATADIR, filename, NULL);
+}
 
+static void
+clutter_test_init (gint    *argc,
+                   gchar ***argv)
+{
 #ifdef HAVE_CLUTTER_GLX
   /* on X11 we need a display connection to run the test suite */
   const gchar *display = g_getenv ("DISPLAY");
@@ -68,7 +76,8 @@ main (int argc, char **argv)
     {
       g_print ("No DISPLAY found. Unable to run the conformance "
                "test suite without a display.");
-      return EXIT_SUCCESS;
+
+      exit (EXIT_SUCCESS);
     }
 #endif
 
@@ -79,17 +88,24 @@ main (int argc, char **argv)
    */
   g_setenv ("CLUTTER_VBLANK", "none", FALSE);
 
-  g_test_init (&argc, &argv, NULL);
+  g_test_init (argc, argv, NULL);
 
   g_test_bug_base ("http://bugzilla.openedhand.com/show_bug.cgi?id=%s");
 
   /* Initialise the state you need to share with everything.
    */
-  shared_state->argc_addr = &argc;
-  shared_state->argv_addr = &argv;
+  shared_state = g_new0 (TestConformSharedState, 1);
+  shared_state->argc_addr = argc;
+  shared_state->argv_addr = argv;
 
   g_assert (clutter_init (shared_state->argc_addr, shared_state->argv_addr)
 	    == CLUTTER_INIT_SUCCESS);
+}
+
+int
+main (int argc, char **argv)
+{
+  clutter_test_init (&argc, &argv);
 
   TEST_CONFORM_SIMPLE ("/timeline", test_timeline);
   TEST_CONFORM_SKIP (!g_test_slow (), "/timeline", test_timeline_interpolate);
@@ -97,7 +113,6 @@ main (int argc, char **argv)
 
   TEST_CONFORM_SIMPLE ("/picking", test_pick);
 
-  /* ClutterText */
   TEST_CONFORM_SIMPLE ("/text", test_text_utf8_validation);
   TEST_CONFORM_SIMPLE ("/text", test_text_empty);
   TEST_CONFORM_SIMPLE ("/text", test_text_set_empty);
@@ -116,8 +131,6 @@ main (int argc, char **argv)
   TEST_CONFORM_SIMPLE ("/rectangle", test_rect_set_size);
   TEST_CONFORM_SIMPLE ("/rectangle", test_rect_set_color);
 
-  TEST_CONFORM_SIMPLE ("/fixed", test_fixed_constants);
-
   TEST_CONFORM_SIMPLE ("/invariants", test_initial_state);
   TEST_CONFORM_SIMPLE ("/invariants", test_shown_not_parented);
   TEST_CONFORM_SIMPLE ("/invariants", test_realized);
@@ -127,37 +140,78 @@ main (int argc, char **argv)
   TEST_CONFORM_SIMPLE ("/invariants", test_show_on_set_parent);
   TEST_CONFORM_SIMPLE ("/invariants", test_clone_no_map);
 
-  TEST_CONFORM_SIMPLE ("/vertex-buffer", test_vertex_buffer_contiguous);
-  TEST_CONFORM_SIMPLE ("/vertex-buffer", test_vertex_buffer_interleved);
-  TEST_CONFORM_SIMPLE ("/vertex-buffer", test_vertex_buffer_mutability);
-
   TEST_CONFORM_SIMPLE ("/opacity", test_label_opacity);
   TEST_CONFORM_SIMPLE ("/opacity", test_rectangle_opacity);
   TEST_CONFORM_SIMPLE ("/opacity", test_paint_opacity);
 
-  TEST_CONFORM_SIMPLE ("/texture", test_backface_culling);
-  TEST_CONFORM_SIMPLE ("/texture", test_npot_texture);
-  TEST_CONFORM_SIMPLE ("/texture", test_premult);
+  TEST_CONFORM_SIMPLE ("/texture", test_texture_fbo);
+  TEST_CONFORM_SIMPLE ("/texture/cairo", test_clutter_cairo_texture);
 
   TEST_CONFORM_SIMPLE ("/path", test_path);
 
   TEST_CONFORM_SIMPLE ("/binding-pool", test_binding_pool);
 
+#if 0
   TEST_CONFORM_SIMPLE ("/actor", test_anchors);
+#endif
+  TEST_CONFORM_SIMPLE ("/actor", test_actor_destruction);
 
   TEST_CONFORM_SIMPLE ("/model", test_list_model_populate);
   TEST_CONFORM_SIMPLE ("/model", test_list_model_iterate);
   TEST_CONFORM_SIMPLE ("/model", test_list_model_filter);
-
-  TEST_CONFORM_SIMPLE ("/material", test_materials);
-  TEST_CONFORM_SIMPLE ("/material", test_blend_strings);
+  TEST_CONFORM_SIMPLE ("/model", test_list_model_from_script);
 
   TEST_CONFORM_SIMPLE ("/color", test_color_from_string);
   TEST_CONFORM_SIMPLE ("/color", test_color_to_string);
   TEST_CONFORM_SIMPLE ("/color", test_color_hls_roundtrip);
+  TEST_CONFORM_SIMPLE ("/color", test_color_operators);
 
   TEST_CONFORM_SIMPLE ("/units", test_units_constructors);
   TEST_CONFORM_SIMPLE ("/units", test_units_string);
+  TEST_CONFORM_SIMPLE ("/units", test_units_cache);
+
+  TEST_CONFORM_SIMPLE ("/group", test_group_depth_sorting);
+
+  TEST_CONFORM_SIMPLE ("/sizing", test_fixed_size);
+  TEST_CONFORM_SIMPLE ("/sizing", test_preferred_size);
+
+  TEST_CONFORM_SIMPLE ("/script", test_script_single);
+  TEST_CONFORM_SIMPLE ("/script", test_script_child);
+  TEST_CONFORM_SIMPLE ("/script", test_script_implicit_alpha);
+  TEST_CONFORM_SIMPLE ("/script", test_script_object_property);
+  TEST_CONFORM_SIMPLE ("/script", test_script_animation);
+  TEST_CONFORM_SIMPLE ("/script", test_script_named_object);
+  TEST_CONFORM_SIMPLE ("/script", test_animator_base);
+  TEST_CONFORM_SIMPLE ("/script", test_animator_properties);
+  TEST_CONFORM_SIMPLE ("/script", test_animator_multi_properties);
+  TEST_CONFORM_SIMPLE ("/script", test_state_base);
+
+  TEST_CONFORM_SIMPLE ("/behaviours", test_behaviours);
+
+  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_fixed);
+  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_backface_culling);
+  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_materials);
+  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_blend_strings);
+  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_premult);
+  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_readpixels);
+  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_path);
+
+  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_npot_texture);
+  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_multitexture);
+  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_texture_mipmaps);
+  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_sub_texture);
+  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_pixel_buffer);
+  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_texture_rectangle);
+  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_wrap_modes);
+
+  TEST_CONFORM_SIMPLE ("/cogl/vertex-buffer", test_cogl_vertex_buffer_contiguous);
+  TEST_CONFORM_SIMPLE ("/cogl/vertex-buffer", test_cogl_vertex_buffer_interleved);
+  TEST_CONFORM_SIMPLE ("/cogl/vertex-buffer", test_cogl_vertex_buffer_mutability);
+
+  /* left to the end because they aren't currently very orthogonal and tend to
+   * break subsequent tests! */
+  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_viewport);
+  TEST_CONFORM_SIMPLE ("/cogl", test_cogl_offscreen);
 
   return g_test_run ();
 }
